<?php
/**
 * Geral\Repositories
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Geral\Repositories;

/**
 * Class Servico
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Servico extends \Doctrine\ORM\EntityRepository
{
    /**
     * Consulta lista de serviço da filial
     *
     * @param String $nrorg          Organização
     * @param String $filial         Unidade
     * @param String $centralMerenda Central de merenda
     *
     * @return array|bool
     */
    public function consultaListaServicoFilial($nrorg, $filial, $centralMerenda)
    {
        $sql
            = "SELECT DISTINCT SE.CDSERVFILI,
                       SE.NMSERVICO
                  FROM FILITIPOCARD FC,
                       SERVTIPOCARD SC,
                       SERVICO SE
                 WHERE SC.NRORG        = :P_NRORG        AND
                       SC.CDFILCENTPME = :P_CDFILCENTPME AND
                       SC.NRORG        = FC.NRORG        AND
                       SC.CDFILCENTPME = FC.CDFILCENTPME AND
                       SC.CDTIPOCARD   = FC.CDTIPOCARD   AND
                       :P_CDFILIAL     = FC.CDFILIAL     AND
                       SC.NRORG        = SE.NRORG        AND
                       SC.CDFILCENTPME = SE.CDFILIAL     AND
                       SC.CDSERVFILI   = SE.CDSERVFILI
                 ORDER BY SE.CDSERVFILI";

        $params = array(
            "P_NRORG" => $nrorg,
            "P_CDFILIAL" => $filial,
            "P_CDFILCENTPME" => $centralMerenda,
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        $result = $em->getConnection()->fetchAll($sql, $params);
        return count($result) > 0 ? $result : false;
    }

    /**
     * Consulta comensais estimados
     * @param String $nrorg   Organização
     * @param String $filial  Unidade
     * @param String $servico Serviço
     * @param String $idDia   Id do Dia
     *
     * @return bool
     */
    public static function consultaComensaisEstimados($nrorg, $filial, $servico, $idDia)
    {
        $sql
            = "SELECT DECODE(:P_TPDIA,
                              1, SERVICO.QTCOMESTIDOM,
                              2, SERVICO.QTCOMESTISEG,
                              3, SERVICO.QTCOMESTITER,
                              4, SERVICO.QTCOMESTIQUA,
                              5, SERVICO.QTCOMESTIQUI,
                              6, SERVICO.QTCOMESTISEX,
                              7, SERVICO.QTCOMESTISAB,
                              8, SERVICO.QTCOMESTIFER) AS COMENSAISEST
                  FROM SERVICO
                 WHERE SERVICO.CDFILIAL = :P_CDFILIAL
                   AND SERVICO.NRORG = :P_NRORG
                   AND SERVICO.CDSERVFILI = :P_CDSERVFILI";

        $params = array(
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_CDSERVFILI' => $servico,
            'P_TPDIA' => ++$idDia,
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        $result = $em->getConnection()->fetchAll($sql, $params);
        return count($result) > 0 ? $result[0]['COMENSAISEST'] : false;
    }

    /**
     * Consulta lista de grupo de serviço
     * @param String $nrorg      Organização
     * @param String $cdfilial   Unidade
     * @param String $cdgrupserv Grupo de serviço
     *
     * @return array|bool
     */
    public static function consultaListaServicoGruposervico($nrorg, $cdfilial, $cdgrupserv)
    {
        $sql
            = "SELECT CDSERVFILI,CDFILIAL
                FROM SERVICO
                WHERE NRORG = :P_NRORG
                AND (INSTR(:P_CDFILIAL,CDFILIAL) <> 0 OR :P_CDFILIAL = 'A')
                AND (INSTR(:P_CDGRUPSERV,CDGRUPSERV) <> 0 OR :P_CDGRUPSERV = 'A')
                AND IDSERVATIV <> 'N'";

        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDFILIAL' => $cdfilial,
            'P_CDGRUPSERV' => $cdgrupserv
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        $result = $em->getConnection()->fetchAll($sql, $params);
        return count($result) > 0 ? $result : false;

    }

    /**
     * Consulta lista de grupo de filial
     * @param String $nrorg      Organização
     * @param String $cdgrupfili Grupo de filial
     *
     * @return array|bool
     */
    public static function consultaListaFilialGrupofilial($nrorg, $cdgrupfili)
    {
        $sql
            = "SELECT *
                FROM
                    (SELECT FILIAL.CDFILIAL,
                    FILIAL.NMFILIAL,
                    FILIAL.NRORG,
                    FILGRUPFILI.CDGRUPFILI
                    FROM FILIAL
                    LEFT JOIN FILGRUPFILI
                    ON FILIAL.NRORG     = FILGRUPFILI.NRORG
                    AND FILIAL.CDFILIAL = FILGRUPFILI.CDFILIAL ORDER BY CDFILIAL
                    ) FILIAL
                WHERE FILIAL.CDGRUPFILI = :P_CDGRUPFILI
                AND FILIAL.NRORG        = :P_NRORG";

        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDGRUPFILI' => $cdgrupfili
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        $result = $em->getConnection()->fetchAll($sql, $params);
        return count($result) > 0 ? $result : false;
    }

    /**
     * Atualizar o serviço correspondentes ao Padrão de Serviço informado
     *
     * @param String $nrorg      Organização
     * @param String $cdfilial   Unidade
     * @param String $tfilial    Todas Unidades
     * @param String $cdproduto  Código do produto
     * @param String $nmservico  Nome do serviço
     * @param String $cdservpadr Código do serviço
     *
     * @return null
     */
    public static function atualizaPadraoServico($nrorg, $cdfilial, $tfilial, $cdproduto, $nmservico, $cdservpadr)
    {
        $sql
            = "UPDATE
                  SERVICO
               SET
                  CDPRODUTO = :P_CDPRODUTO,
                  NMSERVICO = :P_NMSERVPADR
               WHERE
                  ((CDFILIAL = :P_CDFILIAL) OR (:T_CDFILIAL   = 'T'))
                  AND NRORG = :P_NRORG
                  AND CDSERVFILI = :P_CDSERVPADR";

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NMSERVPADR' => $nmservico,
            'P_CDPRODUTO' => $cdproduto,
            'P_CDFILIAL' => $cdfilial,
            'T_CDFILIAL' => $tfilial,
            'P_CDSERVPADR' => $cdservpadr
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        $result = $em->getConnection()->executeUpdate($sql, $params);
    }

    /**
     * Atualiza os parâmetos do serviço
     *
     * @param $cdfiliorig
     * @param $cdservorig
     * @param $listafiliserv
     * @param $hrinicaten
     * @param $hrfinaserv
     * @param $idservativ
     * @param $hrentrega
     * @param $cdserpaddie
     * @param $idtiposerv
     * @param $idconsservca
     * @param $idbloqconfixper
     * @param $idutretiprat
     * @param $idbloqaltpra
     * @param $idconscomfat
     * @param $idservprinc
     * @param $idservtran
     * @param $idvlrvardata
     * @param $idservfatsep
     * @param $idcarddescentr
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function atualizaParametrosServico(
        $cdfiliorig, $cdservorig, $listafiliserv, $hrinicaten, $cdrotatrans,
        $hrfinaserv, $idservativ, $hrentrega, $cdserpaddie, $idtiposerv,
        $idconsservca, $idbloqconfixper, $idutretiprat, $idbloqaltpra,
        $idconscomfat, $idservprinc, $idservtran, $idvlrvardata,
        $idservfatsep, $idcarddescentr
    ) {
        try {
            $sql
                = "UPDATE SERVICO
                    SET HRINICATEN    = :HRINICATEN,
                      CDROTATRANS     = :CDROTATRANS,
                      HRINICSERV      = :HRINICATEN,
                      HRFINASERV      = :HRFINASERV,
                      IDSERVATIV      = :IDSERVATIV,
                      HRENTREGA       = :HRENTREGA,
                      CDSERPADDIE     = :CDSERPADDIE,
                      IDTIPOSERV      = :IDTIPOSERV,
                      IDCONSSERVCA    = :IDCONSSERVCA,
                      IDBLOQCONFIXPER = :IDBLOQCONFIXPER,
                      IDUTRETIPRAT    = :IDUTRETIPRAT,
                      IDBLOQALTPRA    = :IDBLOQALTPRA,
                      IDCONSCOMFAT    = :IDCONSCOMFAT,
                      IDSERVPRINC     = :IDSERVPRINC,
                      IDSERVTRAN      = :IDSERVTRAN,
                      IDVLRVARDATA    = :IDVLRVARDATA,
                      IDSERVFATSEP    = :IDSERVFATSEP,
                      IDCARDDESCENTR  = :IDCARDDESCENTR
                    WHERE ((INSTR(:P_LISTAFILISERV,CDFILIAL||CDSERVFILI) <> 0)
                    OR (:P_LISTAFILISERV = 'T'))
                    AND (CDFILIAL||CDSERVFILI <> :P_CDFILIORIG||:P_CDSERVORIG)";
            $params = array(
                'HRINICATEN' => $hrinicaten,
                'CDROTATRANS' => $cdrotatrans,
                'HRFINASERV' => $hrfinaserv,
                'IDSERVATIV' => $idservativ,
                'HRENTREGA' => $hrentrega,
                'CDSERPADDIE' => $cdserpaddie,
                'IDTIPOSERV' => $idtiposerv,
                'IDCONSSERVCA' => $idconsservca,
                'IDBLOQCONFIXPER' => $idbloqconfixper,
                'IDUTRETIPRAT' => $idutretiprat,
                'IDBLOQALTPRA' => $idbloqaltpra,
                'IDCONSCOMFAT' => $idconscomfat,
                'IDSERVPRINC' => $idservprinc,
                'IDSERVTRAN' => $idservtran,
                'IDVLRVARDATA' => $idvlrvardata,
                'IDSERVFATSEP' => $idservfatsep,
                'IDCARDDESCENTR' => $idcarddescentr,
                'P_LISTAFILISERV' => $listafiliserv,
                'P_CDFILIORIG' => $cdfiliorig,
                'P_CDSERVORIG' => $cdservorig,
            );

            return $this->_em->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Incluir o serviço na exportação da configuração padrão
     *
     * @param $cdfiliorig
     * @param $cdservorig
     * @param $cdfilidest
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function incluirServicoExportacao(
        $cdfiliorig, $cdservorig, $cdfilidest
    ) {
        try {
            $sql
                = "INSERT INTO SERVICO
                    (CDFILIAL,
                    CDSERVFILI,NMSERVICO,CDGRUPSERV,HRINICATEN,QTCOMESTISEG,
                    QTCOMESTITER,QTCOMESTIQUA,QTCOMESTIQUI,QTCOMESTISEX,
                    QTCOMESTISAB,QTCOMESTIDOM,QTCOMESTIFER,QTCLIESTISEG,
                    QTCLIESTITER,QTCLIESTIQUA,QTCLIESTIQUI,QTCLIESTISEX,
                    QTCLIESTISAB,QTCLIESTIDOM,QTCLIESTIFER,QTFUNESTISEG,
                    QTFUNESTITER,QTFUNESTIQUA,QTFUNESTIQUI,QTFUNESTISEX,
                    QTFUNESTISAB,QTFUNESTIDOM,QTFUNESTIFER,QTDOAESTISEG,
                    QTDOAESTITER,QTDOAESTIQUA,QTDOAESTIQUI,QTDOAESTISEX,
                    QTDOAESTISAB,QTDOAESTIDOM,QTDOAESTIFER,CDPRODUTO,IDFATUSERV,
                    IDSERVPADR,IDTIPOSERV,CDROTATRANS,HRINICSERV,HRFINASERV,
                    CDSERVDOS,IDSERVTRAN,VRPESOCUSTO,DTULTATUSERV,CDSERPADDIE,
                    IDTPSERVHOSP,IDCONSSERVCA,IDSERVPRINC,IDUTRETIPRAT,
                    IDSERVATIV,IDBLOQALTPRA,CDSERVPREQUI,IDCONSCOMFAT,
                    IDVLRVARDATA,IDCARDDESCENTR,NMFANTASSERV,IDSERVFATSEP,
                    IDBLOQCONFIXPER,IDUTPRATRELAC,IDINDICADOR,IDCARDCONSUMIDORWEB,
                    HRENTREGA,NMSERVICOREDU)
                (SELECT S.CDFILIAL,
                        :P_CDSERVORIG AS CDSERVFILI,T.NMSERVICO,T.CDGRUPSERV,
                        T.HRINICATEN,T.QTCOMESTISEG,T.QTCOMESTITER,T.QTCOMESTIQUA,
                        T.QTCOMESTIQUI,T.QTCOMESTISEX,T.QTCOMESTISAB,T.QTCOMESTIDOM,
                        T.QTCOMESTIFER,T.QTCLIESTISEG,T.QTCLIESTITER,T.QTCLIESTIQUA,
                        T.QTCLIESTIQUI,T.QTCLIESTISEX,T.QTCLIESTISAB,T.QTCLIESTIDOM,
                        T.QTCLIESTIFER,T.QTFUNESTISEG,T.QTFUNESTITER,T.QTFUNESTIQUA,
                        T.QTFUNESTIQUI,T.QTFUNESTISEX,T.QTFUNESTISAB,T.QTFUNESTIDOM,
                        T.QTFUNESTIFER,T.QTDOAESTISEG,T.QTDOAESTITER,T.QTDOAESTIQUA,
                        T.QTDOAESTIQUI,T.QTDOAESTISEX,T.QTDOAESTISAB,T.QTDOAESTIDOM,
                        T.QTDOAESTIFER,T.CDPRODUTO,T.IDFATUSERV,T.IDSERVPADR,
                        T.IDTIPOSERV,T.CDROTATRANS,T.HRINICSERV,T.HRFINASERV,
                        T.CDSERVDOS,T.IDSERVTRAN,T.VRPESOCUSTO,T.DTULTATUSERV,
                        T.CDSERPADDIE,T.IDTPSERVHOSP,T.IDCONSSERVCA,T.IDSERVPRINC,
                        T.IDUTRETIPRAT,T.IDSERVATIV,T.IDBLOQALTPRA,T.CDSERVPREQUI,
                        T.IDCONSCOMFAT,T.IDVLRVARDATA,T.IDCARDDESCENTR,T.NMFANTASSERV,
                        T.IDSERVFATSEP,T.IDBLOQCONFIXPER,T.IDUTPRATRELAC,
                        T.IDINDICADOR,T.IDCARDCONSUMIDORWEB,T.HRENTREGA,T.NMSERVICOREDU
                FROM SERVICO T, (SELECT F.CDFILIAL
                                    FROM FILIAL F, SERVICO SE
                                   WHERE F.CDFILIAL = SE.CDFILIAL(+)
                                   MINUS
                                  SELECT FF.CDFILIAL
                                    FROM FILIAL FF, SERVICO SS
                                   WHERE FF.CDFILIAL = SS.CDFILIAL(+)
                                     AND SS.CDSERVFILI = :P_CDSERVORIG) S
                WHERE (T.CDFILIAL   = :P_CDFILIORIG)
                AND (T.CDSERVFILI = :P_CDSERVORIG)
                AND ((INSTR(:P_CDFILIDEST,S.CDFILIAL)<>0) OR (:P_CDFILIDEST = 'T')))";
            $params = array(
                'P_CDSERVORIG' => $cdservorig,
                'P_CDFILIORIG' => $cdfiliorig,
                'P_CDFILIDEST' => $cdfilidest,
            );

            $resultado = $this->_em->getConnection()->executeQuery($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getServicosDescentralizados($nrorg, $nrlista)
    {
        $sql
            = "SELECT S.CDSERVFILI
                FROM SERVICO S,
                     LISTA LS
                WHERE S.NRORG        = :P_NRORG
                AND S.NRORG          = LS.NRORG
                AND LS.NRLISTA       = :P_NRLISTASERV
                AND S.CDFILIAL       = SUBSTR(LS.CDLISTA, 1, 4)
                AND S.CDSERVFILI     = SUBSTR(LS.CDLISTA, 5, 5)
                AND S.IDCARDDESCENTR = 'S'";

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRLISTASERV' => $nrlista
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);

        return count($result) > 0 ? $result : false;
    }


    public static function BuscaServicoPorFilial($cdfilial){
        $sql = <<<SQL
SELECT * FROM (SELECT SERVICO.CDSERVFILI,
                                      SERVICO.NMSERVICO,
                                      SERVICO.CDFILIAL,
                                      FILIAL.NMFILIAL,
                                      FILIAL.CDFILIAL
                  FROM SERVICO
                  INNER JOIN FILIAL
                      ON  SERVICO.NRORG    = FILIAL.NRORG
                      AND SERVICO.CDFILIAL = FILIAL.CDFILIAL
                  WHERE
                      ((INSTR(:CDFILIAL,FILIAL.CDFILIAL) <> 0) OR (:CDFILIAL = 'A'))

                ) SERVICO
                ORDER BY NMFILIAL, NMSERVICO
SQL;

        $params = array(
            'CDFILIAL'     =>$cdfilial,
        );

        try{
            $conn = \Setforms2\Shared\Registry::getEntityManager();
            $conn = $conn->getConnection();
            return $conn-> fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function BuscaListaServicoPorFilial($cdfilial, $cdservfili){
        $sql = <<<SQL
SELECT SERVICO.CDSERVFILI,
  SERVICO.NMSERVICO,
  SERVICO.CDFILIAL,
  FILIAL.NMFILIAL
FROM SERVICO
INNER JOIN FILIAL
ON SERVICO.NRORG                             = FILIAL.NRORG
AND SERVICO.CDFILIAL                         = FILIAL.CDFILIAL
WHERE ((INSTR(:CDFILIAL,FILIAL.CDFILIAL)    <> 0)
OR (:CDFILIAL                                = 'A'))
AND ((INSTR(:CDSERVFILI,SERVICO.CDSERVFILI) <> 0)
OR (:CDSERVFILI                              = 'A'))
SQL;
        $params = array(
            'CDFILIAL'     =>$cdfilial,
            'CDSERVFILI'   =>$cdservfili
        );

        try{
            $conn = \Setforms2\Shared\Registry::getEntityManager();
            $conn = $conn->getConnection();
            return $conn-> fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }

    }
}